@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2018-2023 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2018-2025 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -19,15 +19,16 @@
  */
 
 #include <glib.h>
+#include <locale.h>
+
 #include "appstream-compose.h"
 #include "asc-font-private.h"
 #include "asc-utils-metainfo.h"
 #include "asc-utils-l10n.h"
 #include "asc-utils-screenshots.h"
 #include "asc-utils-fonts.h"
-#include "asc-image.h"
+#include "asc-image-private.h"
 #include "asc-canvas.h"
-#include "asc-canvas-private.h"
 
 #include "as-utils-private.h"
 #include "as-test-utils.h"
@@ -147,34 +148,51 @@ test_read_fontinfo (void)
 	g_autofree gchar *data = NULL;
 	gsize data_len;
 	g_autoptr(GList) lang_list = NULL;
-	guint i;
+	const gchar *expected_langs_old_fontconfig[] = {
+		"aa",	  "ab",	   "af",  "an",	 "ast",	  "av",	   "ay",  "az-az", "ba",  "be",
+		"bg",	  "bi",	   "bin", "br",	 "bs",	  "bua",   "ca",  "ce",	   "ch",  "chm",
+		"co",	  "crh",   "cs",  "csb", "cv",	  "cy",	   "da",  "de",	   "en",  "eo",
+		"es",	  "et",	   "eu",  "fi",	 "fil",	  "fj",	   "fo",  "fr",	   "fur", "fy",
+		"gd",	  "gl",	   "gn",  "gv",	 "haw",	  "ho",	   "hr",  "hsb",   "ht",  "hu",
+		"ia",	  "id",	   "ie",  "ig",	 "ik",	  "io",	   "is",  "it",	   "jv",  "kaa",
+		"ki",	  "kj",	   "kk",  "kl",	 "ku-am", "ku-tr", "kum", "kv",	   "kw",  "kwm",
+		"ky",	  "la",	   "lb",  "lez", "lg",	  "li",	   "lt",  "lv",	   "mg",  "mh",
+		"mk",	  "mn-mn", "mo",  "ms",	 "mt",	  "na",	   "nb",  "nds",   "ng",  "nl",
+		"nn",	  "no",	   "nr",  "nso", "nv",	  "ny",	   "oc",  "om",	   "os",  "pap-an",
+		"pap-aw", "pl",	   "pt",  "qu",	 "quz",	  "rm",	   "rn",  "ro",	   "ru",  "rw",
+		"sah",	  "sc",	   "se",  "sel", "sg",	  "sh",	   "sk",  "sl",	   "sm",  "sma",
+		"smj",	  "smn",   "sn",  "so",	 "sq",	  "sr",	   "ss",  "st",	   "su",  "sv",
+		"sw",	  "tg",	   "tk",  "tl",	 "tn",	  "to",	   "tr",  "ts",	   "tt",  "ty",
+		"tyv",	  "uk",	   "uz",  "vi",	 "vo",	  "vot",   "wa",  "wen",   "wo",  "xh",
+		"yap",	  "za",	   "zu",  NULL
+	};
 	const gchar *expected_langs[] = {
-		"aa",  "ab",	 "af",	"ak", "an",	"ast",	  "av",	   "ay",    "az-az", "ba",
-		"be",  "ber-dz", "bg",	"bi", "bin",	"bm",	  "br",	   "bs",    "bua",   "ca",
-		"ce",  "ch",	 "chm", "co", "crh",	"cs",	  "csb",   "cu",    "cv",    "cy",
-		"da",  "de",	 "ee",	"el", "en",	"eo",	  "es",	   "et",    "eu",    "fat",
-		"ff",  "fi",	 "fil", "fj", "fo",	"fr",	  "fur",   "fy",    "ga",    "gd",
-		"gl",  "gn",	 "gv",	"ha", "haw",	"ho",	  "hr",	   "hsb",   "ht",    "hu",
-		"hz",  "ia",	 "id",	"ie", "ig",	"ik",	  "io",	   "is",    "it",    "jv",
-		"kaa", "kab",	 "ki",	"kj", "kk",	"kl",	  "kr",	   "ku-am", "ku-tr", "kum",
-		"kv",  "kw",	 "kwm", "ky", "la",	"lb",	  "lez",   "lg",    "li",    "ln",
-		"lt",  "lv",	 "mg",	"mh", "mi",	"mk",	  "mn-mn", "mo",    "ms",    "mt",
-		"na",  "nb",	 "nds", "ng", "nl",	"nn",	  "no",	   "nr",    "nso",   "nv",
-		"ny",  "oc",	 "om",	"os", "pap-an", "pap-aw", "pl",	   "pt",    "qu",    "quz",
-		"rm",  "rn",	 "ro",	"ru", "rw",	"sah",	  "sc",	   "sco",   "se",    "sel",
-		"sg",  "sh",	 "shs", "sk", "sl",	"sm",	  "sma",   "smj",   "smn",   "sms",
-		"sn",  "so",	 "sq",	"sr", "ss",	"st",	  "su",	   "sv",    "sw",    "tg",
-		"tk",  "tl",	 "tn",	"to", "tr",	"ts",	  "tt",	   "tw",    "ty",    "tyv",
-		"uk",  "uz",	 "ve",	"vi", "vo",	"vot",	  "wa",	   "wen",   "wo",    "xh",
-		"yap", "yo",	 "za",	"zu", NULL
+		"aa",	  "ab",	    "af",  "agr", "an",	 "ast", "av",  "ay",  "ayc",   "az-az",
+		"ba",	  "be",	    "bem", "bg",  "bi",	 "bin", "br",  "bs",  "bua",   "ca",
+		"ce",	  "ch",	    "chm", "co",  "crh", "cs",	"csb", "cv",  "cy",    "da",
+		"de",	  "dsb",    "en",  "eo",  "es",	 "et",	"eu",  "fi",  "fil",   "fj",
+		"fo",	  "fr",	    "fur", "fy",  "gd",	 "gl",	"gn",  "gv",  "haw",   "ho",
+		"hr",	  "hsb",    "ht",  "hu",  "ia",	 "id",	"ie",  "ig",  "ik",    "io",
+		"is",	  "it",	    "jv",  "kaa", "ki",	 "kj",	"kk",  "kl",  "ku-am", "ku-tr",
+		"kum",	  "kv",	    "kw",  "kwm", "ky",	 "la",	"lb",  "lez", "lg",    "li",
+		"lij",	  "lt",	    "lv",  "mfe", "mg",	 "mh",	"mhr", "miq", "mjw",   "mk",
+		"mn-mn",  "mo",	    "ms",  "mt",  "na",	 "nb",	"nds", "ng",  "nhn",   "niu",
+		"nl",	  "nn",	    "no",  "nr",  "nso", "nv",	"ny",  "oc",  "om",    "os",
+		"pap-an", "pap-aw", "pl",  "pt",  "qu",	 "quz", "rm",  "rn",  "ro",    "ru",
+		"rw",	  "sah",    "sc",  "se",  "sel", "sg",	"sgs", "sh",  "sk",    "sl",
+		"sm",	  "sma",    "smj", "smn", "sn",	 "so",	"sq",  "sr",  "ss",    "st",
+		"su",	  "sv",	    "sw",  "szl", "tg",	 "tk",	"tl",  "tn",  "to",    "tpi",
+		"tr",	  "ts",	    "tt",  "ty",  "tyv", "uk",	"unm", "uz",  "vi",    "vo",
+		"vot",	  "wa",	    "wae", "wen", "wo",	 "xh",	"yap", "yuw", "za",    "zu",
+		NULL
 	};
 
-	font_fname = g_build_filename (datadir, "NotoSans-Regular.ttf", NULL);
+	font_fname = g_build_filename (datadir, "Raleway-Regular.ttf", NULL);
 
 	/* test reading from file */
 	font = asc_font_new_from_file (font_fname, &error);
 	g_assert_no_error (error);
-	g_assert_cmpstr (asc_font_get_family (font), ==, "Noto Sans");
+	g_assert_cmpstr (asc_font_get_family (font), ==, "Raleway");
 	g_assert_cmpstr (asc_font_get_style (font), ==, "Regular");
 	g_object_unref (font);
 	font = NULL;
@@ -183,31 +201,54 @@ test_read_fontinfo (void)
 	g_file_get_contents (font_fname, &data, &data_len, &error);
 	g_assert_no_error (error);
 
-	font = asc_font_new_from_data (data, data_len, "NotoSans-Regular.ttf", &error);
+	font = asc_font_new_from_data (data, data_len, "Raleway-Regular.ttf", &error);
 	g_assert_no_error (error);
-	g_assert_cmpstr (asc_font_get_family (font), ==, "Noto Sans");
+	g_assert_cmpstr (asc_font_get_family (font), ==, "Raleway");
 	g_assert_cmpstr (asc_font_get_style (font), ==, "Regular");
 	g_assert_cmpint (asc_font_get_charset (font), ==, FT_ENCODING_UNICODE);
-	g_assert_cmpstr (asc_font_get_homepage (font), ==, "http://www.monotype.com/studio");
+	g_assert_cmpstr (asc_font_get_homepage (font), ==, "http://pixelspread.com");
 	g_assert_cmpstr (asc_font_get_description (font),
 			 ==,
-			 "Data hinted. Designed by Monotype design team.");
+			 "Raleway is an elegant sans-serif typeface family. "
+			 "Initially designed by Matt McInerney as a single thin weight, "
+			 "it was expanded into a 9 weight family by Pablo Impallari and "
+			 "Rodrigo Fuenzalida in 2012 and iKerned by Igino Marini. "
+			 "It is a display face and the download features both old style "
+			 "and lining numerals, standard and discretionary ligatures, a "
+			 "pretty complete set of diacritics, as well as a stylistic "
+			 "alternate inspired by more geometric sans-serif typefaces "
+			 "than its neo-grotesque inspired default character set.");
 
 	lang_list = asc_font_get_language_list (font);
-	i = 0;
-	for (GList *l = lang_list; l != NULL; l = l->next) {
-		g_assert_true (expected_langs[i] != NULL);
-		g_assert_cmpstr (expected_langs[i], ==, l->data);
-		i++;
+
+	{
+		guint i = 0;
+		gboolean fc_lang_success = TRUE;
+		for (GList *l = lang_list; l != NULL; l = l->next) {
+			g_assert_nonnull (expected_langs_old_fontconfig[i]);
+			if (!as_str_equal0 (expected_langs_old_fontconfig[i], l->data)) {
+				fc_lang_success = FALSE;
+				break;
+			}
+			i++;
+		}
+		if (!fc_lang_success) {
+			i = 0;
+			for (GList *l = lang_list; l != NULL; l = l->next) {
+				g_assert_nonnull (expected_langs[i]);
+				g_assert_cmpstr (expected_langs[i], ==, l->data);
+				i++;
+			}
+		}
 	}
 
 	/* uses "Noto Sans" */
 	g_assert_cmpstr (asc_font_get_sample_text (font),
 			 ==,
-			 "My grandfather picks up quartz and valuable onyx jewels.");
-	g_assert_cmpstr (asc_font_find_pangram (font, "en", "Noto Sans"),
+			 "A mad boxer shot a quick, gloved jab to the jaw of his dizzy opponent.");
+	g_assert_cmpstr (asc_font_find_pangram (font, "en", "Raleway"),
 			 ==,
-			 "My grandfather picks up quartz and valuable onyx jewels.");
+			 "A mad boxer shot a quick, gloved jab to the jaw of his dizzy opponent.");
 
 	g_assert_cmpstr (asc_font_find_pangram (font, "en", "aaaaa"),
 			 ==,
@@ -227,6 +268,7 @@ test_image_transform (void)
 {
 	g_autoptr(GHashTable) supported_fmts = NULL;
 	g_autofree gchar *sample_img_fname = NULL;
+	g_autofree gchar *sample_jxl_img_fname = NULL;
 	g_autoptr(AscImage) image = NULL;
 	g_autoptr(GError) error = NULL;
 	gboolean ret;
@@ -241,9 +283,14 @@ test_image_transform (void)
 	g_assert_true (g_hash_table_contains (supported_fmts, "jpeg"));
 
 	sample_img_fname = g_build_filename (datadir, "appstream-logo.png", NULL);
+	sample_jxl_img_fname = g_build_filename (datadir, "image.jxl", NULL);
 
 	/* load image from file */
-	image = asc_image_new_from_file (sample_img_fname, 0, ASC_IMAGE_LOAD_FLAG_NONE, &error);
+	image = asc_image_new_from_file (sample_img_fname,
+					 -1,
+					 -1,
+					 ASC_IMAGE_LOAD_FLAG_NONE,
+					 &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (image);
 
@@ -264,8 +311,7 @@ test_image_transform (void)
 	g_assert_no_error (error);
 	g_assert_true (ret);
 
-	g_object_unref (image);
-	image = NULL;
+	g_clear_object (&image);
 
 	/* test reading image from memory */
 	g_file_get_contents (sample_img_fname, &data, &data_len, &error);
@@ -273,9 +319,10 @@ test_image_transform (void)
 
 	image = asc_image_new_from_data (data,
 					 data_len,
-					 0,
-					 FALSE,
+					 -1,
+					 -1,
 					 ASC_IMAGE_LOAD_FLAG_NONE,
+					 ASC_IMAGE_FORMAT_UNKNOWN,
 					 &error);
 	g_assert_no_error (error);
 	g_assert_nonnull (image);
@@ -289,6 +336,26 @@ test_image_transform (void)
 				       &error);
 	g_assert_no_error (error);
 	g_assert_true (ret);
+	g_clear_object (&image);
+
+	/* test loading a JPEG-XL image */
+	image = asc_image_new_from_file (sample_jxl_img_fname,
+					 -1,
+					 -1,
+					 ASC_IMAGE_LOAD_FLAG_NONE,
+					 &error);
+	if (g_hash_table_contains (supported_fmts, "jxl")) {
+		g_assert_no_error (error);
+		g_assert_nonnull (image);
+
+		g_assert_cmpint (asc_image_get_width (image), ==, 64);
+		g_assert_cmpint (asc_image_get_height (image), ==, 64);
+	} else {
+		g_assert_error (error, ASC_IMAGE_ERROR, ASC_IMAGE_ERROR_UNSUPPORTED);
+		g_assert_null (image);
+		g_clear_error (&error);
+	}
+	g_clear_object (&image);
 }
 
 /**
@@ -303,6 +370,9 @@ test_canvas (void)
 	g_autofree gchar *font_fname = NULL;
 	g_autofree gchar *data = NULL;
 	gsize data_len;
+	gint cv_size;
+	gint text_border_width, shape_border_width;
+	AscCanvasShape bg_shape;
 	g_autoptr(AscCanvas) cv = NULL;
 	g_autoptr(AscFont) font = NULL;
 	g_autoptr(GInputStream) stream = NULL;
@@ -327,7 +397,7 @@ test_canvas (void)
 	cv = NULL;
 
 	/* test font rendering */
-	font_fname = g_build_filename (datadir, "NotoSans-Regular.ttf", NULL);
+	font_fname = g_build_filename (datadir, "Raleway-Regular.ttf", NULL);
 	font = asc_font_new_from_file (font_fname, &error);
 	g_assert_no_error (error);
 
@@ -344,6 +414,67 @@ test_canvas (void)
 
 	asc_canvas_save_png (cv, "/tmp/asc-fontrender_test1.png", &error);
 	g_assert_no_error (error);
+	g_object_unref (cv);
+
+	cv_size = 128;
+	bg_shape = ASC_CANVAS_SHAPE_CVL_TRIANGLE;
+	shape_border_width = (gint) (cv_size * 0.032);
+	text_border_width = asc_calculate_text_border_width_for_icon_shape (bg_shape,
+									    cv_size,
+									    shape_border_width);
+
+	cv = asc_canvas_new (cv_size, cv_size);
+	asc_canvas_draw_shape (cv,
+			       bg_shape,
+			       shape_border_width,
+			       0.84, /* red */
+			       0.84, /* green */
+			       0.84, /* blue */
+			       &error);
+	g_assert_no_error (error);
+
+	asc_canvas_draw_text_line (cv,
+				   font,
+				   "Aa",
+				   text_border_width,
+				   bg_shape == ASC_CANVAS_SHAPE_CVL_TRIANGLE
+				       ? (gint) ((cv_size / 2.0 - shape_border_width) * 0.15)
+				       : 0,
+				   &error);
+	g_assert_no_error (error);
+	asc_canvas_save_png (cv, "/tmp/asc-fontrender_test2.png", &error);
+	g_assert_no_error (error);
+}
+
+/**
+ * test_render_font_card:
+ *
+ * Test font-card rendering on canvas.
+ */
+static void
+test_render_font_card (void)
+{
+	g_autofree gchar *font_fname = NULL;
+	g_autoptr(AscFont) font = NULL;
+	g_autoptr(AscCanvas) canvas = NULL;
+	g_autoptr(GError) error = NULL;
+
+	font_fname = g_build_filename (datadir, "Raleway-Regular.ttf", NULL);
+	font = asc_font_new_from_file (font_fname, &error);
+	g_assert_no_error (error);
+
+	canvas = asc_canvas_new (800, 600);
+	asc_canvas_draw_font_card (canvas,
+				   font,
+				   NULL, /* default info label */
+				   asc_font_find_pangram (font, "en", "Raleway"), /* pangram */
+				   NULL, /* default bg letter */
+				   -1,	 /* default margin    */
+				   &error);
+	g_assert_no_error (error);
+
+	asc_canvas_save_png (canvas, "/tmp/asc-font-card.png", &error);
+	g_assert_no_error (error);
 }
 
 /**
@@ -473,14 +604,14 @@ test_compose_desktop_entry (void)
 	gchar *tmp;
 	AsLaunchable *launch;
 	GPtrArray *hints;
-	g_autoptr(GBytes) de_bytes = as_gbytes_from_literal (
-	    "[Desktop Entry]\n"
-	    "Type=Application\n"
-	    "Name=FooBar\n"
-	    "Name[de_DE]=FööBär\n"
-	    "Comment=A foo-ish bar.\n"
-	    "Keywords=Hobbes;Bentham;Locke;\n"
-	    "Keywords[de_DE]=Heidegger;Kant;Hegel;\n");
+	g_autoptr(GBytes)
+		      de_bytes = as_gbytes_from_literal ("[Desktop Entry]\n"
+							 "Type=Application\n"
+							 "Name=FooBar\n"
+							 "Name[de_DE]=FööBär\n"
+							 "Comment=A foo-ish bar.\n"
+							 "Keywords=Hobbes;Bentham;Locke;\n"
+							 "Keywords[de_DE]=Heidegger;Kant;Hegel;\n");
 
 	cres = asc_result_new ();
 
@@ -663,11 +794,11 @@ test_compose_directory_unit (void)
 	g_assert_true (ret);
 
 	contents = asc_unit_get_contents (ASC_UNIT (dirunit));
-	g_assert_cmpint (contents->len, ==, 15);
+	g_assert_cmpint (contents->len, ==, 17);
 	as_sort_strings (contents);
 
-	g_assert_cmpstr (g_ptr_array_index (contents, 0), ==, "/Noto.LICENSE");
-	g_assert_cmpstr (g_ptr_array_index (contents, 5), ==, "/table.svgz");
+	g_assert_cmpstr (g_ptr_array_index (contents, 0), ==, "/Raleway-Regular.ttf");
+	g_assert_cmpstr (g_ptr_array_index (contents, 5), ==, "/sample-video.mkv");
 
 	/* read existent data */
 	g_assert_true (asc_unit_file_exists (ASC_UNIT (dirunit), "/usr/dummy"));
@@ -909,11 +1040,49 @@ test_compose_font (void)
 	asc_assert_no_hints_in_result (cres);
 }
 
+static void
+test_compose_icon_policy_serialize (void)
+{
+	g_autoptr(AscIconPolicy) ipolicy = NULL;
+	g_autofree gchar *tmp = NULL;
+	gboolean ret;
+	g_autoptr(GError) error = NULL;
+
+	ipolicy = asc_icon_policy_new ();
+	tmp = asc_icon_policy_to_string (ipolicy);
+	g_assert_cmpstr (tmp,
+			 ==,
+			 "48x48=cached,48x48@2=cached,64x64=cached,64x64@2=cached,"
+			 "128x128=cached-remote,128x128@2=cached-remote");
+	g_free (g_steal_pointer (&tmp));
+
+	ret = asc_icon_policy_from_string (
+	    ipolicy,
+	    "48x48@2=ignored,64x64=cached,64x64@2=cached-remote,128x128=remote,128x128@2=remote",
+	    &error);
+	g_assert_no_error (error);
+	g_assert_true (ret);
+	g_clear_error (&error);
+
+	tmp = asc_icon_policy_to_string (ipolicy);
+	g_assert_cmpstr (
+	    tmp,
+	    ==,
+	    "48x48@2=ignored,64x64=cached,64x64@2=cached-remote,128x128=remote,128x128@2=remote");
+	g_free (g_steal_pointer (&tmp));
+
+	ret = asc_icon_policy_from_string (ipolicy, "48x48-2:ignored,64x64:cached", &error);
+	g_assert_error (error, AS_UTILS_ERROR, AS_UTILS_ERROR_FAILED);
+	g_assert_false (ret);
+}
+
 int
 main (int argc, char **argv)
 {
 	int ret;
 
+	setlocale (LC_ALL, "");
+
 	if (argc == 0) {
 		g_error ("No test directory specified!");
 		return 1;
@@ -940,6 +1109,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/AppStream/Compose/FontInfo", test_read_fontinfo);
 	g_test_add_func ("/AppStream/Compose/Image", test_image_transform);
 	g_test_add_func ("/AppStream/Compose/Canvas", test_canvas);
+	g_test_add_func ("/AppStream/Compose/FontCard", test_render_font_card);
 	g_test_add_func ("/AppStream/Compose/Hints", test_compose_hints);
 	g_test_add_func ("/AppStream/Compose/Result", test_compose_result);
 	g_test_add_func ("/AppStream/Compose/DesktopEntry", test_compose_desktop_entry);
@@ -948,6 +1118,8 @@ main (int argc, char **argv)
 	g_test_add_func ("/AppStream/Compose/SourceLocale", test_compose_source_locale);
 	g_test_add_func ("/AppStream/Compose/VideoInfo", test_compose_video_info);
 	g_test_add_func ("/AppStream/Compose/Font", test_compose_font);
+	g_test_add_func ("/AppStream/Compose/IconPolicySerialize",
+			 test_compose_icon_policy_serialize);
 
 	ret = g_test_run ();
 	g_free (datadir);
